import tictacpy

class Emojis:
    def __init__(self, x, o, b):
        self.x = x
        self.o = o
        self.b = b

class TTT:
    def __init__(self, xPlayer, oPlayer, x:str = "❌", o:str = "⭕", b:str = "ㅤ"):
        if xPlayer == oPlayer:
            raise tictacpy.CannotBeTheSame("The players for x and o cannot be the same!")
        self.board = tictacpy.Board(x, o, b)
        self.emojis = Emojis(x, o, b)
        self.xPlayer = xPlayer
        self.oPlayer = oPlayer
        self.turnPlayer = xPlayer
    
    def turn(self, space:str, player:str):
        if space.lower() not in ("a1", "b1", "c1", "a2", "b2", "c2", "a3", "b3", "c3"):
            raise tictacpy.InvalidSquare(F"Square {space} is not on the board!")
        else:
            if player.lower() not in ("x", "o"):
                raise tictacpy.InvalidPlayer("The player must be an \"x\" or an \"o\"")
            elif player == "x":
                turnplayer = self.xPlayer
            elif player == "o":
                turnplayer = self.oPlayer
            
            if turnplayer != self.turnPlayer:
                raise tictacpy.NotTheirTurn("It is not that player's turn!")
            
            square = self.board.decode(space)
            if square is None:
                raise tictacpy.UnknownError("An unknown error has occured!")
            else:
                if square != self.emojis.b:
                    return False
                else:
                    if turnplayer == self.xPlayer:
                        self.turnPlayer = self.oPlayer
                    else:
                        self.turnPlayer = self.xPlayer
                    return self.board.change(space, player, self.emojis.x, self.emojis.o)
    
    def is_over(self):
        if self.board.A1 == self.board.A2 and self.board.A2 == self.board.A3 and self.board.A1 == self.emojis.x:
            return "X"
        elif self.board.A1 == self.board.A2 and self.board.A2 == self.board.A3 and self.board.A1 == self.emojis.o:
            return "O"
        elif self.board.B1 == self.board.B2 and self.board.B2 == self.board.B3 and self.board.B1 == self.emojis.x:
            return "X"
        elif self.board.B1 == self.board.B2 and self.board.B2 == self.board.B3 and self.board.B1 == self.emojis.o:
            return "O"
        elif self.board.C1 == self.board.C2 and self.board.C2 == self.board.C3 and self.board.C1 == self.emojis.x:
            return "X"
        elif self.board.C1 == self.board.C2 and self.board.C2 == self.board.C3 and self.board.C1 == self.emojis.o:
            return "O"
        
        elif self.board.A1 == self.board.B1 and self.board.B1 == self.board.C1 and self.board.A1 == self.emojis.x:
            return "X"
        elif self.board.A1 == self.board.B1 and self.board.B1 == self.board.C1 and self.board.A1 == self.emojis.o:
            return "O"
        elif self.board.A2 == self.board.B2 and self.board.B2 == self.board.C2 and self.board.A2 == self.emojis.x:
            return "X"
        elif self.board.A2 == self.board.B2 and self.board.B2 == self.board.C2 and self.board.A2 == self.emojis.o:
            return "O"
        elif self.board.A3 == self.board.B3 and self.board.B3 == self.board.C3 and self.board.A3 == self.emojis.x:
            return "X"
        elif self.board.A3 == self.board.B3 and self.board.B3 == self.board.C3 and self.board.A3 == self.emojis.o:
            return "O"
        
        elif self.board.A1 == self.board.B2 and self.board.B2 == self.board.C3 and self.board.A1 == self.emojis.x:
            return "X"
        elif self.board.A1 == self.board.B2 and self.board.B2 == self.board.C3 and self.board.A1 == self.emojis.o:
            return "O"
        elif self.board.A3 == self.board.B2 and self.board.B2 == self.board.C1 and self.board.A3 == self.emojis.x:
            return "X"
        elif self.board.A3 == self.board.B2 and self.board.B2 == self.board.C1 and self.board.A3 == self.emojis.o:
            return "O"
        
        elif self.board.A1 != self.emojis.b and self.board.A2 != self.emojis.b and self.board.A3 != self.emojis.b and self.board.B1 != self.emojis.b and self.board.B2 != self.emojis.b and self.board.B3 != self.emojis.b and self.board.C1 != self.emojis.b and self.board.C2 != self.emojis.b and self.board.C3 != self.emojis.b:
            return None
        
        else:
            return False
    
    def visualize(self):
        return "\n".join([
            F"{self.board.A1}|{self.board.A2}|{self.board.A3}",
            F"{self.board.B1}|{self.board.B2}|{self.board.B3}",
            F"{self.board.C1}|{self.board.C2}|{self.board.C3}"
        ])


class InvalidSquare(Exception):
    pass

class InvalidPlayer(Exception):
    pass

class UnknownError(Exception):
    pass

class NotTheirTurn(Exception):
    pass

class CannotBeTheSame(Exception):
    pass

class SpaceOccupied(Exception):
    pass


import tictacpy

class Board:
    def __init__(self, x, o, b):
        self.A1 = b
        self.A2 = b
        self.A3 = b
        self.B1 = b
        self.B2 = b
        self.B3 = b
        self.C1 = b
        self.C2 = b
        self.C3 = b
    
    def decode(self, square:str):
        if square.lower() not in ("a1", "b1", "c1", "a2", "b2", "c2", "a3", "b3", "c3"):
            raise tictacpy.InvalidSquare(F"Square {square} is not on the board!")
        elif square.lower() == "a1":
            return self.A1
        elif square.lower() == "a2":
            return self.A2
        elif square.lower() == "a3":
            return self.A3
        elif square.lower() == "b1":
            return self.B1
        elif square.lower() == "b2":
            return self.B2
        elif square.lower() == "b3":
            return self.B3
        elif square.lower() == "c1":
            return self.C1
        elif square.lower() == "c2":
            return self.C2
        elif square.lower() == "c3":
            return self.C3
        else:
            return None
        
    def change(self, square, emoji, x, o):
        if emoji not in ("x", "o"):
            raise tictacpy.InvalidPlayer("Your player must be \"x\" or \"o\"")
        if square.lower() not in ("a1", "b1", "c1", "a2", "b2", "c2", "a3", "b3", "c3"):
            raise tictacpy.InvalidSquare(F"Square {square} is not on the board!")
        elif square.lower() == "a1":
            if emoji == "x":
                self.A1 = x
            elif emoji == "o":
                self.A1 = o
            else:
                return False
            return True
        elif square.lower() == "a2":
            if emoji == "x":
                self.A2 = x
            elif emoji == "o":
                self.A2 = o
            else:
                return False
            return True
        elif square.lower() == "a3":
            if emoji == "x":
                self.A3 = x
            elif emoji == "o":
                self.A3 = o
            else:
                return False
            return True
        elif square.lower() == "b1":
            if emoji == "x":
                self.B1 = x
            elif emoji == "o":
                self.B1 = o
            else:
                return False
            return True
        elif square.lower() == "b2":
            if emoji == "x":
                self.B2 = x
            elif emoji == "o":
                self.B2 = o
            else:
                return False
            return True
        elif square.lower() == "b3":
            if emoji == "x":
                self.B3 = x
            elif emoji == "o":
                self.B3 = o
            else:
                return False
            return True
        elif square.lower() == "c1":
            if emoji == "x":
                self.C1 = x
            elif emoji == "o":
                self.C1 = o
            else:
                return False
            return True
        elif square.lower() == "c2":
            if emoji == "x":
                self.C2 = x
            elif emoji == "o":
                self.C2 = o
            else:
                return False
            return True
        elif square.lower() == "c3":
            if emoji == "x":
                self.C3 = x
            elif emoji == "o":
                self.C3 = o
            else:
                return False
            return True
        else:
            return False